import argparse
import os
from mukham.detector import detect_largest_face


"""
    Argument parser for command line interface
"""
my_parser = argparse.ArgumentParser(description="mukham: Crop the largest face from an image.")
my_parser.add_argument('-i', '--input', type=str, required=True, help='path to input image file')
my_parser.add_argument('-o', '--output', type=str, required=False, help='path to save output image file')
my_parser.add_argument(
    '-c', '--conf', type=float, required=False, 
    help='confidence threshold for face detected',
    default=0.8
    )

# parse arguments
args =  my_parser.parse_args()

# run
bound_box = detect_largest_face(args.input, out_path=args.output, min_conf=args.conf)
print(bound_box)


import unittest
import os
import cv2


class MukhamTest(unittest.TestCase):
    """
        Tests for the required functionalities:
        1. Dimension Error: allow only images of max size 1024x1024
        2. Bounding box for the largest face must be returned
        3. save the largest file to an appropriate location. 
    """
    
    def setUp(self):
        """Executed before the any tests start."""
        # remove existing test files
        if os.path.isfile('data/faces/bryant_daughter.jpg'):
            os.remove(' data/faces/bryant_daughter.jpg')
    
    def test_DimensionError(self):
        """Test if DimensionError is raised"""
        from mukham.detector import DimensionError, detect_largest_face
        self.assertRaises(DimensionError, detect_largest_face, 'data/images/kobe_large.jpg')
    
    def test_detector(self):
        """Test if the detector works."""
        from mukham.detector import detect_largest_face
        bounding_box = detect_largest_face('data/images/bryant_daughter.jpg')
        img = cv2.imread('data/images/bryant_daughter.jpg')
        self.assertTrue(bounding_box[0][0] < img.shape[1])
        self.assertTrue(bounding_box[1][0] < img.shape[1])
        self.assertTrue(bounding_box[0][1] < img.shape[0])
        self.assertTrue(bounding_box[1][1] < img.shape[0])
    
    def test_save_file(self):
        """Test if the detector saves the file."""
        from mukham.detector import detect_largest_face
        # detect largest face
        detect_largest_face(
            'data/images/bryant_daughter.jpg', 
            out_path='data/faces/bryant_daughter.jpg'
            )
        # test if saved
        self.assertTrue(os.path.isfile('data/faces/bryant_daughter.jpg'))

if __name__ == "__main__":
    unittest.main()


import numpy as np
import cv2
import errno

# set environment variable
import os
os.environ['OPENCV_IO_ENABLE_JASPER']= 'TRUE'   # allows JPEG2000 format


class DimensionError(Exception):
    """
        raised when the image does not meet the required 
        maximum dimensions of 1024 x 1024.
    """
    def __init__(self, h, w):
        message = "Image is too big " + str((h, w))
        message += "; max allowed size is (1024, 1024)"
        super(DimensionError, self).__init__(message)


def detect_largest_face(in_path, out_path=None, min_conf=0.8):
    """
        Detects largest face using the DNN face detection algorithm
        from cv2 library.

        Args:
            in_path (str): path of the input image file
            out_path (str, optional): path of the cropped image file. Defaults to None
            min_conf (float, optional): threshold confidence to detect face. Defaults to 0.8
        
        Returns:
            bounding_box: an 2x2 array of two (x,y) coordinates 
                for top left and bottom right of the bounding box
    """

    # check input file path
    if not os.path.isfile(in_path):
        raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), in_path)
    
    # check output file path
    if out_path:
        try:
            with open(out_path, 'w') as f:
                pass
        
        except OSError:
            raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), out_path)

    # read file
    img = cv2.imread(in_path)

    # check image dimensions
    h, w = img.shape[:2]
    if h > 1024 or w > 1024:
        raise DimensionError(h, w)

    # detect faces using DNN algorithm from cv2
    net = cv2.dnn.readNetFromCaffe(
        "models/deploy.prototxt",
        "/models/res10_300x300_ssd_iter_140000.caffemodel"
        )

    rgb_mean = np.mean(img, axis=(0, 1)) # mean rgb values to remove effects of illumination
    blob = cv2.dnn.blobFromImage(cv2.resize(img, (300,300)), 1.0, (300, 300), rgb_mean)
    net.setInput(blob)
    faces = net.forward()

    # get the most confident faces
    conf_faces = np.array(list(filter(lambda x: x[2] > min_conf, faces[0, 0])))

    # check if any faces exist
    assert len(conf_faces) > 0, "No faces found!"

    # get the largest face
    first_face = 0 # let first face be biggest face
    first_box = conf_faces[first_face, 3:7] * np.array([w, h, w, h])
    sx, sy, ex, ey = first_box.astype("int")

    for i in range(1, conf_faces.shape[0]):
        box = conf_faces[i, 3:7] * np.array([w, h, w, h])
        (startX, startY, endX, endY) = box.astype("int")
        
        if (endX - startX)*(endY - startY) > (ex - sx)*(ey - sy):
            sx, sy, ex, ey = startX, startY, endX, endY

    # save the crop
    if out_path:
        largest_crop = img[sy:ey, sx:ex]
        saved_file = cv2.imwrite(out_path, largest_crop)
    
    # return the largest bounding box
    bounding_box = [(sx, sy), (ex, ey)]
    return bounding_box